diff --git a/src/backend/access/transam/xlog.c b/src/backend/access/transam/xlog.c
index 187b609..e962f62 100644
--- a/src/backend/access/transam/xlog.c
+++ b/src/backend/access/transam/xlog.c
@@ -5923,6 +5923,10 @@ recoveryStopsHere(XLogRecord *record, bool *includeThis)
 static void
 recoveryPausesHere(void)
 {
+	/* Don't pause unless users can connect! */
+	if (!LocalHotStandbyActive)
+		return;
+
 	ereport(LOG,
 			(errmsg("recovery has paused"),
 			 errhint("Execute pg_xlog_replay_resume() to continue.")));
@@ -6697,19 +6701,18 @@ StartupXLOG(void)
 				 */
 				if (recoveryStopsHere(record, &recoveryApply))
 				{
-					/*
-					 * Pause only if users can connect to send a resume
-					 * message
-					 */
-					if (recoveryPauseAtTarget && standbyState == STANDBY_SNAPSHOT_READY)
-					{
+					if (recoveryPauseAtTarget)
 						SetRecoveryPause(true);
-						recoveryPausesHere();
-					}
+
 					reachedStopPoint = true;	/* see below */
 					recoveryContinue = false;
+
+					/* break out if we have non-inclusive recovery target */
 					if (!recoveryApply)
+					{
+						recoveryPausesHere();
 						break;
+					}
 				}
 
 				/* Setup error traceback support for ereport() */
@@ -6737,19 +6740,14 @@ StartupXLOG(void)
 				/*
 				 * Update shared replayEndRecPtr before replaying this record,
 				 * so that XLogFlush will update minRecoveryPoint correctly.
+				 *
+				 * While we have the lock, also check for a pause request.
 				 */
 				SpinLockAcquire(&xlogctl->info_lck);
 				xlogctl->replayEndRecPtr = EndRecPtr;
-				recoveryPause = xlogctl->recoveryPause;
 				SpinLockRelease(&xlogctl->info_lck);
 
 				/*
-				 * Pause only if users can connect to send a resume message
-				 */
-				if (recoveryPause && standbyState == STANDBY_SNAPSHOT_READY)
-					recoveryPausesHere();
-
-				/*
 				 * If we are attempting to enter Hot Standby mode, process
 				 * XIDs we see
 				 */
@@ -6790,10 +6788,19 @@ StartupXLOG(void)
 				 */
 				SpinLockAcquire(&xlogctl->info_lck);
 				xlogctl->recoveryLastRecPtr = EndRecPtr;
+				recoveryPause = xlogctl->recoveryPause;
 				SpinLockRelease(&xlogctl->info_lck);
 
 				LastRec = ReadRecPtr;
 
+
+				if (recoveryPause)
+					recoveryPausesHere();
+
+				/* if we have inclusive recovery target */
+				if (!recoveryStopsHere)
+					break;
+
 				record = ReadRecord(NULL, LOG, false);
 			} while (record != NULL && recoveryContinue);
 
